@@ -1,6 +1,10 @@
-import sys
-import string
+import math
 import secrets
+import sys
+
+from . import strings
+from .cli_args import ArgParser
+from .contants import *
 
 
 class Charset:
@@ -82,6 +86,7 @@ def generate(
     l_lower=True,
     charsets=[],
     charset_file="",
+    output_file="",
     **kwargs,
 ):
     """Generates a secrets password based on the arguments.
@@ -97,6 +102,7 @@ def generate(
         l_lower (bool, optional): Whether to use lowercase letters or not. Defaults to True.
         charsets (list, optional): The charsets that will be used instead of the arguments specification. Defaults to [].
         charset_file (str, optional): The charset file will be used instead of the arguments specification. Defaults to "".
+        output_file (str, optional): The output file will be created with the passwords.
 
     Returns:
         str | list: return a list if the quantity is greater than 1
@@ -133,33 +139,80 @@ def generate(
 
     else:
         if punctuation:
-            password_gen.add_charset(Charset(string.punctuation, length))
+            password_gen.add_charset(Charset(strings.punctuation, length))
         if digits:
-            password_gen.add_charset(Charset(string.digits, length))
+            password_gen.add_charset(Charset(strings.digits, length))
         if letters:
             if l_upper:
-                password_gen.add_charset(Charset(string.ascii_uppercase, length))
+                password_gen.add_charset(Charset(strings.ascii_uppercase, length))
             if l_lower:
-                password_gen.add_charset(Charset(string.ascii_lowercase, length))
+                password_gen.add_charset(Charset(strings.ascii_lowercase, length))
     if quantity > 1:
         password_list = []
         for _ in range(quantity):
             password_list.append(password_gen.generate_password())
-        return password_list
+        ret = password_list
+    else:
+        ret = password_gen.generate_password()
 
-    return password_gen.generate_password()
+    if output_file:
+        _export_passwords(output_file, ret)
+    else:
+        return ret
 
 
-def test(password: str) -> str:
-    ...
+def _export_passwords(output_file, passwords):
+    with open(output_file, "w") as file:
+        if isinstance(passwords, str):
+            passwords = [passwords]
+        for pwd in passwords[:-1]:
+            file.writelines(pwd + "\n")
+        file.writelines(pwd)
+
+
+def entropy(password: str) -> float:
+    if not isinstance(password, str):
+        raise TypeError("password must be a string")
+
+    lowercase = False
+    uppercase = False
+    digits = False
+    punctuation = False
+    ascii_extended = False
+
+    for letter in set(password):
+        if letter in strings.ascii_lowercase:
+            lowercase = True
+        elif letter in strings.ascii_uppercase:
+            uppercase = True
+        elif letter in strings.digits:
+            digits = True
+        elif letter in strings.punctuation:
+            punctuation = True
+        elif letter in strings.ascii_extended:
+            ascii_extended = True
+
+    pool_size = 0
+    if lowercase:
+        pool_size += len(strings.ascii_lowercase)
+    if uppercase:
+        pool_size += len(strings.ascii_uppercase)
+    if digits:
+        pool_size += len(strings.digits)
+    if punctuation:
+        pool_size += len(strings.punctuation)
+    if ascii_extended:
+        pool_size += len(strings.ascii_extended)
+
+    # Shannon
+    return round(len(password) * math.log(pool_size, 2), 2)
 
 
 def main():
-    from . import cli_args
 
-    args = cli_args.get_args()
+    args = ArgParser.get_args()
     try:
-        if args.command in cli_args.GENERATE:
+        if args.command in GENERATE:
             print(
                 generate(
                     args.quantity,
@@ -170,10 +223,11 @@ def main():
                     l_upper=args.upper,
                     l_lower=args.lower,
                     charset_file=args.charset_file,
+                    output_file=args.output,
                 )
             )
-        elif args.command in cli_args.TEST:
-            print(test(args.password))
+        elif args.command in ENTROPY:
+            print(entropy(args.password))
 
     except Exception as error_msg:
         __show_error(error_msg.args[-1])
